# -*- coding: utf-8 -*-
# Copyright (c) 2019  Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Written by Chenxiong Qi <cqi@redhat.com>

import pytest

from mock import Mock, patch
from tests import make_mmd, make_mbs_response, URSA_MAJOR_TEST_CONFIG
from ursa_major.handlers.base import BaseHandler
from ursa_major import ModuleConfig


class TestBaseHanlder:

    # NOTE: once MBS supports filtering modules builds by requires or
    # buildrequires, the test data should be updated because it is unnecessary
    # to filter module builds locally by Ursa-Major itself.

    @pytest.mark.parametrize(
        'module_config,module_builds,expected_koji_tags', [
            # None of these koji_tags exist in tag inheritance.
            (ModuleConfig({'name': 'mariadb', 'stream': '10.3',
                           'version': '3120190304180825', 'context': 'f636be4b'}),
             [{'id': 1,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.3-3020190301191548-a5b0195c'},
              {'id': 2,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.3-3120190304180825-f636be4b'}],
             []),

            # koji_tag exists in tag inheritance
            (ModuleConfig({'name': 'mariadb', 'stream': '10.4',
                           'version': '3120190304180825', 'context': 'f636be4b'}),
             [{'id': 1,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.4-3020190313091759-a5b0195c'},
              {'id': 2,
               'name': 'mariadb',
               # This older koji_tag was added previously
               'koji_tag': 'module-mariadb-10.4-3020190304180835-a5b0195c'}],
             ['module-mariadb-10.4-3020190304180835-a5b0195c']),

            # Modules are filtered by requires.
            # None of these koji_tags exist in tag inheritance.
            (ModuleConfig({'name': 'mariadb', 'stream': '10.3',
                           'version': '3120190304180825', 'context': 'f636be4b',
                           'requires': {'platform': 'f30'}}),
             [{'id': 1,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.3-3020190301191548-a5b0195c',
               'modulemd': make_mmd(name='mariadb', stream='10.3',
                                    version='3020190301191548', context='a5b0195c',
                                    requires={'platform': 'f30'},
                                    buildrequires={'platform': 'f30'}).dumps(),
               },
              {'id': 2,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.3-3120190304180825-f636be4b',
               'modulemd': make_mmd(name='mariadb', stream='10.3',
                                    version='3120190304180825', context='f636be4b',
                                    requires={'platform': 'f29'},
                                    buildrequires={'platform': 'f29'}).dumps(),
               }],
             []),

            # koji_tag exists in tag inheritance and module builds are
            # filtered by both requires and buildrequires.
            (ModuleConfig({'name': 'mariadb', 'stream': '10.4',
                           'version': '3120190304180825', 'context': 'f636be4b',
                           'requires': {'platform': 'f30'},
                           'buildrequires': {'platform': 'f30'}}),
             [{'id': 1,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.4-3020190313091759-a5b0195c',
               'modulemd': make_mmd(name='mariadb', stream='10.4',
                                    version='3020190313091759', context='a5b0195c',
                                    requires={'platform': 'f29'},
                                    buildrequires={'platform': 'f29'}).dumps(),
               },
              {'id': 2,
               'name': 'mariadb',
               # This is the koji_tag for platform:f30
               'koji_tag': 'module-mariadb-10.4-3020190304180835-a5b0195c',
               'modulemd': make_mmd(name='mariadb', stream='10.4',
                                    version='3020190304180835', context='a5b0195c',
                                    requires={'platform': 'f30'},
                                    buildrequires={'platform': 'f30'}).dumps(),
               }],
             ['module-mariadb-10.4-3020190304180835-a5b0195c']),

            # No module builds match the platform:f30
            (ModuleConfig({'name': 'mariadb', 'stream': '10.4',
                           'version': '3120190304180825', 'context': 'f636be4b',
                           'requires': {'platform': 'f30'},
                           'buildrequires': {'platform': 'f30'}}),
             [{'id': 1,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.4-3020190313091759-a5b0195c',
               'modulemd': make_mmd(name='mariadb', stream='10.4',
                                    version='3020190313091759', context='a5b0195c',
                                    requires={'platform': 'f29'},
                                    buildrequires={'platform': 'f29'}).dumps(),
               },
              # Although this koji_tag was added to tag inheritance, but no
              # koji_tag is found that matches platform:f30.
              {'id': 2,
               'name': 'mariadb',
               'koji_tag': 'module-mariadb-10.4-3020190304180835-a5b0195c',
               'modulemd': make_mmd(name='mariadb', stream='10.4',
                                    version='3020190304180835', context='a5b0195c',
                                    requires={'platform': 'f28'},
                                    buildrequires={'platform': 'f28'}).dumps(),
               }],
             []),
        ])
    @patch('ursa_major.koji_service.koji')
    @patch('requests.get')
    def test_find_module_tags_from_koji_inheritance(
            self, get, koji, module_config, module_builds, expected_koji_tags):
        koji_session = koji.ClientSession.return_value
        koji_session.getInheritanceData.return_value = [
            {'parent_id': 100,
             'name': 'module-ant-1.10-20181122140939-819b5873'},
            {'parent_id': 101,
             'name': '"module-hyperfine-latest-3120190318171218-f636be4b'},
            {'parent_id': 102,
             'name': 'module-exa-latest-2820190316115637-8de94f32'},
            {'parent_id': 103,
             'name': 'module-mariadb-10.4-3020190304180835-a5b0195c'},
        ]

        get.return_value = Mock(status_code=200)
        get.return_value.json.return_value = make_mbs_response(module_builds)

        handler = BaseHandler(config=URSA_MAJOR_TEST_CONFIG)
        handler.connect_koji()
        handler.connect_mbs()
        found_koji_tags = handler.find_module_tags_from_koji_inheritance(
            'f30-build', module_config)
        assert expected_koji_tags == found_koji_tags
